/*
 * @(#)AutoAdjustLabelUI.java	1.00    27 Apr 1999
 * 
 * Modification Log:
 *
 */


package aajlabel;

import javax.swing.plaf.metal.MetalLabelUI;
import javax.swing.JLabel;
import javax.swing.Icon;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Insets;
import javax.swing.JComponent;


/**
 * The UI class used by AutoAdjustLabel.<br>
 * 
 * Extends from MetalLabelUI. To implement any other UI, change this and 
 * do corresponding changes to the paint method.<br>
 * 
 * If text won't fit into the available space, first reduces font size to 
 * a minimum value as defined by iReducedFontSize and tries. If it still 
 * won't fit, the text is clipped and the string "..." is added to the end.<br>
 * 
 * Font size 11 was chosen because the default font size of most JVMs is 12 and
 * 10 becomes too small a font size.
 * <br>
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author      Tanmay K. Mohapatra
 * @version     1.00, 27 Apr 1999
 */

public class AutoAdjustLabelUI extends MetalLabelUI
{
    static int  iReducedFontSize = 11;

    
    /* These rectangles/insets are allocated once for this shared LabelUI
     * implementation.  Re-using rectangles rather than allocating
     * them in each paint call halved the time it took paint to run.
     */
    private static Rectangle paintIconR = new Rectangle();
    private static Rectangle paintTextR = new Rectangle();
    private static Rectangle paintViewR = new Rectangle();
    private static Insets paintViewInsets = new Insets(0, 0, 0, 0);

    /** 
     * Paint the label text in the foreground color, if the label
     * is opaque then paint the entire background with the background
     * color.  The Label text is drawn by paintEnabledText() or
     * paintDisabledText().  The locations of the label parts are computed
     * by layoutCL.
     * 
     * @see #paintEnabledText
     * @see #paintDisabledText
     * @see #layoutCL
     */
    public void paint(Graphics g, JComponent c)
    {
        JLabel label = (JLabel)c;
        String text = label.getText();
        Icon icon = (label.isEnabled()) ? label.getIcon() : label.getDisabledIcon();

        if ((icon == null) && (text == null)) {
            return;
        }

        FontMetrics fm = g.getFontMetrics();
        paintViewInsets = c.getInsets(paintViewInsets);

        paintViewR.x = paintViewInsets.left;
        paintViewR.y = paintViewInsets.top;
        paintViewR.width = c.getWidth() - (paintViewInsets.left + paintViewInsets.right);
        paintViewR.height = c.getHeight() - (paintViewInsets.top + paintViewInsets.bottom);

        paintIconR.x = paintIconR.y = paintIconR.width = paintIconR.height = 0;
        paintTextR.x = paintTextR.y = paintTextR.width = paintTextR.height = 0;

        String clippedText = 
            layoutCL(label, fm, text, icon, paintViewR, paintIconR, paintTextR);

        if (!clippedText.equals(text))
        {
            // reduce font size and try again
            Font f = fm.getFont();
            if (f.getSize() > iReducedFontSize)
            {
                f = new Font(f.getFamily(), f.getStyle(), iReducedFontSize);
                if (null != f)
                {
                    g.setFont(f);
                    fm = g.getFontMetrics();
                    clippedText = layoutCL(label, fm, text, icon, paintViewR, paintIconR, paintTextR);
                }
            }
        }
        
        if (icon != null) 
        {
            icon.paintIcon(c, g, paintIconR.x, paintIconR.y);
        }

        if (text != null) 
        {
            int textX = paintTextR.x;
            int textY = paintTextR.y + fm.getAscent();

            if (label.isEnabled()) 
            {
                paintEnabledText(label, g, clippedText, textX, textY);
            }
            else 
            {
                paintDisabledText(label, g, clippedText, textX, textY);
            }
        }
    }    
}
